#include <Adafruit_GFX.h> // Include core graphics library for the display
#include <Adafruit_SSD1306.h> // Include Adafruit_SSD1306 library to drive the display
#include <Fonts/FreeMonoBold12pt7b.h> // Add a custom font
#include <Fonts/FreeMono9pt7b.h> // Add a custom font
#include <MQUnifiedsensor.h>
#include <TimerOne.h>

/************************Hardware Related Macros************************************/
#define Board ("Arduino UNO")
#define Pin (A0)  // Analog input 3 of your Arduino

/***********************Software Related Macros************************************/
#define Type ("MQ-2") // MQ2
#define Voltage_Resolution (5)
#define ADC_Bit_Resolution (10) // For Arduino UNO/MEGA/NANO
#define RatioMQ2CleanAir (9.83) // RS / R0 = 9.83 ppm 

// GLOBALS
Adafruit_SSD1306 display(128, 64); // Create display
MQUnifiedsensor MQ2(Board, Voltage_Resolution, ADC_Bit_Resolution, Pin, Type);
volatile float sensorValue;

// PINS
int buzzer = 2; // Buzzer is connected to D2
int LED1 = 11; // LED 1 is connected to ~D11
int LED2 = 6; // LED 2 is connected to ~D6
int LED3 = 5; // LED 3 is connected to ~D5
int LED4 = 3; // LED 4 is connected to ~D3
int FAN = 13; // FAN is connected to D13

float mapfloat(float x, float in_min, float in_max, float out_min, float out_max)
{
  if (x > in_max) return out_max;
  if (x < in_min) return out_min;
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}

void LEDs_Brightness()
{
  // Map the sensor value to the brightness of the LEDs
  float led1_brightness = mapfloat(sensorValue, 0, 10.0 / 4.0, 0, 255);
  float led2_brightness = mapfloat(sensorValue, 10.0 / 4.0, 10.0 / 2.0, 0, 255);
  float led3_brightness = mapfloat(sensorValue, 10.0 / 2.0, 10.0 / 2.0 + 10.0 / 4.0, 0, 255);
  float led4_brightness = mapfloat(sensorValue, 10.0 / 2.0 + 10.0 / 4.0, 10.0, 0, 255);

  analogWrite(LED1, led1_brightness);
  analogWrite(LED2, led2_brightness);
  analogWrite(LED3, led3_brightness);
  analogWrite(LED4, led4_brightness);
}

void check_sensor()
{
  MQ2.update();
  sensorValue = MQ2.readSensor();

  LEDs_Brightness();

  if (sensorValue > 10)
  {
    digitalWrite(buzzer, HIGH);
    digitalWrite(FAN, HIGH);
  }
  else
  {
    digitalWrite(buzzer, LOW);
    if (sensorValue < 5)
      digitalWrite(FAN, LOW);
  }
}

void setup()
{
  Serial.begin(9600); // For communication with the sensor

  pinMode(buzzer, OUTPUT); // Set buzzer as output

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C); // Initialize display with the I2C address of 0x3C
  display.clearDisplay(); // Clear the buffer
  display.setTextColor(WHITE); // Set color of the text
  display.setRotation(0); // Set orientation
  display.setTextWrap(false); // Set text wrap
  display.dim(0); // Set brightness

  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);
  pinMode(FAN, OUTPUT);

  // Set math model to calculate the PPM concentration and the value of constants
  MQ2.setRegressionMethod(1); // _PPM =  a * ratio^b
  MQ2.setA(574.25); MQ2.setB(-2.222); // Configure the equation to calculate LPG concentration

  // MQ Init
  MQ2.init();
  MQ2.setRL(10);

  // MQ Calibration
  float calcR0 = 0;
  for (int i = 1; i <= 10; i++)
  {
    MQ2.update(); // Update data, the Arduino will read the voltage from the analog pin
    calcR0 += MQ2.calibrate(RatioMQ2CleanAir);
  }
  MQ2.setR0(calcR0 / 10);

  MQ2.update();
  sensorValue = MQ2.readSensor();

  Timer1.initialize(500000); // Set timer to call interrupt every 500ms
  Timer1.attachInterrupt(check_sensor); // Attach the check_sensor function to the timer
}

void set_display()
{
  char string[10]; // Create a character array of 10 characters

  // Convert float to a string
  dtostrf(sensorValue, 3, 0, string); // (<variable>,<amount of digits we are going to use>,<amount of decimal digits>,<string name>)

  display.clearDisplay(); // Clear the display so we can refresh

  display.setFont(&FreeMono9pt7b); // Set a custom font
  display.setTextSize(0); // Set text size. We are using a custom font so you should always use the text size of 0

  // Print text
  display.setCursor(0, 10); // (x,y)
  display.println("Sensor input:"); // Text or value to print

  display.setFont(&FreeMonoBold12pt7b); // Set a custom font
  display.setCursor(50, 50); // (x,y)
  display.println(sensorValue); // Text or value to print

  display.display(); // Print everything we set previously
}

void loop()
{
  set_display(); // Update the display in the main loop
  delay(500); // Add a delay to reduce the update rate of the display
}
